@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -146,9 +146,6 @@ void CAddEdit_Basic::DoDataExchange(CDataExchange* pDX)
     DDX_Control(pDX, IDC_STATIC_URL, m_stc_URL);
     DDX_Control(pDX, IDC_STATIC_EMAIL, m_stc_email);
   }
-
-  if (M_uicaller() == IDS_EDITENTRY && M_protected() != 0)
-    DDX_Control(pDX, IDC_STATIC_PROTECTED, m_stc_protected);
   //}}AFX_DATA_MAP
 }
 
@@ -159,6 +156,7 @@ BEGIN_MESSAGE_MAP(CAddEdit_Basic, CAddEdit_PropertyPage)
 
   ON_BN_CLICKED(IDC_SHOWPASSWORD, OnShowPassword)
   ON_BN_CLICKED(IDC_GENERATEPASSWORD, OnGeneratePassword)
+  ON_BN_CLICKED(IDC_COPYPASSWORD, OnCopyPassword)
   ON_BN_CLICKED(IDC_LAUNCH, OnLaunch)
   ON_BN_CLICKED(IDC_SENDEMAIL, OnSendEmail)
   ON_BN_CLICKED(IDC_VIEWDEPENDENTS, OnViewDependents)
@@ -201,8 +199,9 @@ BOOL CAddEdit_Basic::OnInitDialog()
 
   ModifyStyleEx(0, WS_EX_CONTROLPARENT);
 
-  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD));
-  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD2));
+  Fonts *pFonts = Fonts::GetInstance();
+  pFonts->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD));
+  pFonts->ApplyPasswordFont(GetDlgItem(IDC_PASSWORD2));
 
   // Need to get change notifcations
   m_ex_notes.SetEventMask(ENM_CHANGE | m_ex_notes.GetEventMask());
@@ -210,11 +209,13 @@ BOOL CAddEdit_Basic::OnInitDialog()
   // Set plain text - not that it seems to do much!
   m_ex_notes.SetTextMode(TM_PLAINTEXT);
 
-  m_ex_notes.SetFont(Fonts::GetInstance()->GetCurrentFont());
+  PWSprefs *prefs = PWSprefs::GetInstance();
 
-  if (M_uicaller() == IDS_EDITENTRY && M_protected() != 0) {
-    GetDlgItem(IDC_STATIC_PROTECTED)->ShowWindow(SW_SHOW);
-    m_stc_protected.SetColour(RGB(255,0,0));
+  // Set Notes font!
+  if (prefs->GetPref(PWSprefs::NotesFont).empty()) {
+    m_ex_notes.SetFont(pFonts->GetCurrentFont());
+  } else {
+    m_ex_notes.SetFont(pFonts->GetNotesFont());
   }
 
   if (M_uicaller() != IDS_ADDENTRY) {
@@ -226,12 +227,13 @@ BOOL CAddEdit_Basic::OnInitDialog()
     AddTool(IDC_STATIC_PASSWORD, IDS_CLICKTOCOPY);
     AddTool(IDC_STATIC_NOTES,    IDS_CLICKTOCOPY);
     AddTool(IDC_STATIC_URL,      IDS_CLICKTOCOPY);
+    AddTool(IDC_COPYPASSWORD,    IDS_CLICKTOCOPY);
     AddTool(IDC_STATIC_EMAIL,    IDS_CLICKTOCOPYPLUS1);
     AddTool(IDC_LAUNCH,          IDS_CLICKTOGOPLUS);
     AddTool(IDC_SENDEMAIL,       IDS_CLICKTOSEND);
 
     if (M_uicaller() == IDS_EDITENTRY && M_protected() != 0) {
-      AddTool(IDC_STATIC_PROTECTED, IDS_UNPROTECT);
+      AddTool(IDC_STATIC_TUTORIAL, IDS_UNPROTECT);
     }
 
     ActivateToolTip();
@@ -288,6 +290,9 @@ BOOL CAddEdit_Basic::OnInitDialog()
     m_ex_group.AddString(iter->c_str());
   }
 
+  // Make sure Group combobox is wide enough
+  SetGroupComboBoxWidth();
+
   size_t num_unkwn(0);
   if (M_uicaller() != IDS_ADDENTRY)
     num_unkwn = M_pci()->NumberUnknownFields();
@@ -319,22 +324,23 @@ BOOL CAddEdit_Basic::OnInitDialog()
                        IDS_ISANALIASBASE : IDS_ISASHORTCUTBASE);
     GetDlgItem(IDC_STATIC_ISANALIAS)->SetWindowText(cs_text);
     GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_MYBASE)->ShowWindow(SW_HIDE);
   } else if (M_original_entrytype() == CItemData::ET_ALIAS) {
     // Update password to alias form
     // Show text stating that it is an alias
     M_realpassword() = M_oldRealPassword() = m_password = m_password2 = M_base();
     GetDlgItem(IDC_VIEWDEPENDENTS)->ShowWindow(SW_HIDE);
 
-    cs_text.Format(IDS_ISANALIAS, M_base());
-    GetDlgItem(IDC_STATIC_ISANALIAS)->SetWindowText(cs_text);
+    GetDlgItem(IDC_MYBASE)->SetWindowText(M_base());
     GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_MYBASE)->ShowWindow(SW_SHOW);
   } else if (M_original_entrytype() == CItemData::ET_NORMAL) {
     // Normal - do none of the above
     GetDlgItem(IDC_VIEWDEPENDENTS)->ShowWindow(SW_HIDE);
     GetDlgItem(IDC_STATIC_ISANALIAS)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_MYBASE)->ShowWindow(SW_HIDE);
   }
 
-  PWSprefs *prefs = PWSprefs::GetInstance();
   if (prefs->GetPref(PWSprefs::ShowPWDefault)) {
     ShowPassword();
   } else {
@@ -360,6 +366,21 @@ BOOL CAddEdit_Basic::OnInitDialog()
   m_ex_notes.SetTargetDevice(NULL, m_bWordWrap ? 0 : 1);
   m_ex_notes.UpdateState(PWS_MSG_EDIT_WORDWRAP, m_bWordWrap);
 
+  // Load copy password bitmap
+  UINT nImageID = PWSprefs::GetInstance()->GetPref(PWSprefs::UseNewToolbar) ?
+    IDB_COPYPASSWORD_NEW : IDB_COPYPASSWORD_CLASSIC;
+  BOOL brc = m_CopyPswdBitmap.Attach(::LoadImage(
+                                                 ::AfxFindResourceHandle(MAKEINTRESOURCE(nImageID), RT_BITMAP),
+                                                 MAKEINTRESOURCE(nImageID), IMAGE_BITMAP, 0, 0,
+                                                 (LR_DEFAULTSIZE | LR_CREATEDIBSECTION | LR_SHARED)));
+  ASSERT(brc);
+  if (brc) {
+    FixBitmapBackground(m_CopyPswdBitmap);
+    CButton *pBtn = (CButton *)GetDlgItem(IDC_COPYPASSWORD);
+    ASSERT(pBtn != NULL);
+    if (pBtn != NULL)
+      pBtn->SetBitmap(m_CopyPswdBitmap);
+  }
   UpdateData(FALSE);
   m_bInitdone = true;
   return TRUE;
@@ -401,9 +422,6 @@ HBRUSH CAddEdit_Basic::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
       case IDC_STATIC_EMAIL:
         pcfOld = &m_email_cfOldColour;
         break;
-      case IDC_STATIC_PROTECTED:
-        pcfOld = &m_protected_cfOldColour;
-        break;
       default:
         // Not one of ours - get out quick
         return hbr;
@@ -816,9 +834,14 @@ void CAddEdit_Basic::OnGeneratePassword()
     GetMainDlg()->GetPolicyFromName(M_policyname(), st_pp);
     GetMainDlg()->MakeRandomPassword(passwd, st_pp);
   } else {
-    // XXX temp - to be cleaned up
     PWPolicy policy(M_pwp());
-    policy.symbols = LPCWSTR(M_symbols());
+    if (M_symbols().IsEmpty()) {
+      // No specifc entry symbols - use default
+      policy.symbols = PWSprefs::GetInstance()->GetPref(PWSprefs::DefaultSymbols);
+    } else {
+      // This entry has its own list of symbols
+      policy.symbols = LPCWSTR(M_symbols());
+    }
     GetMainDlg()->MakeRandomPassword(passwd, policy);
   }
 
@@ -1041,12 +1064,55 @@ LRESULT CAddEdit_Basic::OnCallExternalEditor(WPARAM, LPARAM)
 UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
 {
   CAddEdit_Basic *self = (CAddEdit_Basic *)me;
-  CGeneralMsgBox gmb;
 
   wchar_t szExecName[MAX_PATH + 1];
   wchar_t lpPathBuffer[4096];
   DWORD dwBufSize(4096);
 
+  StringX sxEditor = PWSprefs::GetInstance()->GetPref(PWSprefs::AltNotesEditor);
+  if (sxEditor.empty()) {
+    // Find out the users default editor for "txt" files
+    DWORD dwSize(MAX_PATH);
+    HRESULT stat = ::AssocQueryString(0, ASSOCSTR_EXECUTABLE, L".txt", L"Open",
+      szExecName, &dwSize);
+    if (int(stat) != S_OK) {
+#ifdef _DEBUG
+      CGeneralMsgBox gmb;
+      gmb.AfxMessageBox(L"oops");
+#endif
+      // Send error return (WPARAM != 0)
+      self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
+      return 16;
+    }
+    sxEditor = szExecName;
+  }
+
+  DWORD dwResult = ExpandEnvironmentStrings(sxEditor.c_str(), szExecName, MAX_PATH + 1);
+  if (dwResult == 0 || dwResult > (MAX_PATH + 1)) {
+    CGeneralMsgBox gmb;
+    CString cs_msg, cs_title(MAKEINTRESOURCE(IDS_EDITEXTERNALLY));
+    cs_msg.Format(IDS_CANT_FIND_EXT_EDITOR, sxEditor.c_str());
+    gmb.MessageBox(cs_msg, cs_title, MB_OK | MB_ICONEXCLAMATION);
+
+    // Send error return (WPARAM != 0)
+    self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
+    return 0;
+  }
+
+  sxEditor = szExecName;
+
+  if (!pws_os::FileExists(sxEditor.c_str())) {
+    CGeneralMsgBox gmb;
+    CString cs_msg, cs_title(MAKEINTRESOURCE(IDS_EDITEXTERNALLY));
+    cs_msg.Format(IDS_CANT_FIND_EXT_EDITOR, sxEditor.c_str());
+    gmb.MessageBox(cs_msg, cs_title, MB_OK | MB_ICONEXCLAMATION);
+
+    // Send error return (WPARAM != 0)
+    self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
+    return 0;
+  }
+
+  // Now we know the editor exists - go copy the data for it!
   // Get the temp path
   GetTempPath(dwBufSize,          // length of the buffer
               lpPathBuffer);      // buffer for path
@@ -1060,7 +1126,7 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
   FILE *fd;
 
   if ((fd = pws_os::FOpen(self->m_szTempName, _T("w+b"))) == NULL) {
-    return false;
+    return 0;
   }
 
   // Write BOM
@@ -1074,21 +1140,6 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
   // Close file before invoking editor
   fclose(fd);
 
-  StringX sxEditor = PWSprefs::GetInstance()->GetPref(PWSprefs::AltNotesEditor);
-  if (sxEditor.empty()) {
-    // Find out the users default editor for "txt" files
-    DWORD dwSize(MAX_PATH);
-    HRESULT stat = ::AssocQueryString(0, ASSOCSTR_EXECUTABLE, L".txt", L"Open",
-                                      szExecName, &dwSize);
-    if (int(stat) != S_OK) {
-#ifdef _DEBUG
-      gmb.AfxMessageBox(L"oops");
-#endif
-      return 16;
-    }
-    sxEditor = szExecName;
-  }
-
   // Create an Edit process
   STARTUPINFO si;
   PROCESS_INFORMATION pi;
@@ -1109,10 +1160,16 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
 
   if (!CreateProcess(NULL, pszCommandLine, NULL, NULL, FALSE, dwCreationFlags,
                      NULL, lpPathBuffer, &si, &pi)) {
-    pws_os::Trace(L"CreateProcess failed (%d).\n", GetLastError());
+    pws_os::IssueError(L"External Editor CreateProcess", false);
+    CGeneralMsgBox gmb;
+    gmb.AfxMessageBox(IDS_CANT_FIND_EXT_EDITOR, MB_OK | MB_ICONEXCLAMATION);
+
     // Delete temporary file
     _wremove(self->m_szTempName);
     SecureZeroMemory(self->m_szTempName, sizeof(self->m_szTempName));
+
+    // Send error return (WPARAM != 0)
+    self->PostMessage(PWS_MSG_EXTERNAL_EDITOR_ENDED, 16, 0);
     return 0;
   }
 
@@ -1131,20 +1188,24 @@ UINT CAddEdit_Basic::ExternalEditorThread(LPVOID me) // static method!
   return 0;
 }
 
-LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM, LPARAM)
+LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM wParam, LPARAM)
 {
+  std::wstring note;
   GetDlgItem(IDC_NOTES)->EnableWindow(TRUE);
 
-  // Now get what the user saved in this file and put it back into Notes field
-  M_realnotes().Empty();
-  std::wstring note;
+  if (wParam != 0) {
+    goto error_exit;
+  }
 
+  // Now get what the user saved in this file and put it back into Notes field
   FILE *fd;
 
   if ((fd = pws_os::FOpen(m_szTempName, _T("r+b"))) == NULL) {
-    return 0L;
+    goto error_exit;
   }
 
+  M_realnotes().Empty();
+
   ulong64 flength = pws_os::fileLength(fd);
 
   ASSERT(flength % 2 == 0); // guess this is 'cause we assume editor saves wchar_t?
@@ -1211,6 +1272,18 @@ LRESULT CAddEdit_Basic::OnExternalEditorEnded(WPARAM, LPARAM)
 
   OnENChangeNotes();
   return 0L;
+
+error_exit:
+  // Restore notes field and get out
+  m_notes = M_realnotes();
+
+  UpdateData(FALSE);
+  GetDlgItem(IDC_NOTES)->Invalidate();
+
+  // Restore Sheet buttons
+  GetParent()->GetDlgItem(IDOK)->EnableWindow(m_bOKSave == 0 ? TRUE : FALSE);
+  GetParent()->GetDlgItem(IDCANCEL)->EnableWindow(m_bOKCancel == 0 ? TRUE : FALSE);
+  return 0L;
 }
 
 void CAddEdit_Basic::OnViewDependents()
@@ -1354,6 +1427,7 @@ bool CAddEdit_Basic::CheckNewPassword(const StringX &group, const StringX &title
   }
 
   if (pl.ibasedata > 0) {
+    CGeneralMsgBox gmbx;
     if (pl.TargetType == CItemData::ET_ALIAS) {
       // If user tried to point to an alias -> change to point to the 'real' base
       CString cs_msg;
@@ -1361,7 +1435,7 @@ bool CAddEdit_Basic::CheckNewPassword(const StringX &group, const StringX &title
                     pl.csPwdGroup.c_str(),
                     pl.csPwdTitle.c_str(),
                     pl.csPwdUser.c_str());
-      if (gmb.AfxMessageBox(cs_msg, NULL, MB_YESNO | MB_DEFBUTTON2) == IDNO) {
+      if (gmbx.AfxMessageBox(cs_msg, NULL, MB_YESNO | MB_DEFBUTTON2) == IDNO) {
         return false;
       }
     } else {
@@ -1372,7 +1446,7 @@ bool CAddEdit_Basic::CheckNewPassword(const StringX &group, const StringX &title
                       pl.csPwdGroup.c_str(),
                       pl.csPwdTitle.c_str(), 
                       pl.csPwdUser.c_str());
-        gmb.AfxMessageBox(cs_msg, NULL, MB_OK);
+        gmbx.AfxMessageBox(cs_msg, NULL, MB_OK);
         return false;
       } else {
         return true;
@@ -1381,3 +1455,49 @@ bool CAddEdit_Basic::CheckNewPassword(const StringX &group, const StringX &title
   }
   return true; // All OK
 }
+
+void CAddEdit_Basic::SetGroupComboBoxWidth()
+{
+  // Find the longest string in the combo box.
+  CString str;
+  CSize sz;
+  int dx = 0;
+  TEXTMETRIC tm;
+  CDC *pDC = m_ex_group.GetDC();
+  CFont *pFont = m_ex_group.GetFont();
+
+  // Select the listbox font, save the old font
+  CFont *pOldFont = pDC->SelectObject(pFont);
+
+  // Get the text metrics for avg char width
+  pDC->GetTextMetrics(&tm);
+
+  for (int i = 0; i < m_ex_group.GetCount(); i++) {
+    m_ex_group.GetLBText(i, str);
+    sz = pDC->GetTextExtent(str);
+
+    // Add the avg width to prevent clipping
+    sz.cx += tm.tmAveCharWidth;
+
+    if (sz.cx > dx)
+      dx = sz.cx;
+  }
+
+  // Select the old font back into the DC
+  pDC->SelectObject(pOldFont);
+  m_ex_group.ReleaseDC(pDC);
+
+  // Adjust the width for the vertical scroll bar and the left and right border.
+  dx += ::GetSystemMetrics(SM_CXVSCROLL) + 2 * ::GetSystemMetrics(SM_CXEDGE);
+
+  // Set the width of the list box so that every item is completely visible.
+  m_ex_group.SetDroppedWidth(dx);
+}
+
+void CAddEdit_Basic::OnCopyPassword()
+{
+  UpdateData(TRUE);
+
+  GetMainDlg()->SetClipboardData(m_password);
+  GetMainDlg()->UpdateLastClipboardAction(CItemData::PASSWORD);
+}
