@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -30,6 +30,7 @@
 #include "DDStatic.h"
 #include "MenuShortcuts.h"
 #include "AdvancedDlg.h"
+#include "FontsDialog.h"
 
 #include "core/UIinterface.h"
 #include "core/PWScore.h"
@@ -65,10 +66,6 @@
 
 #endif  /* WINVER < 0x0501 */
 
-#if (_MFC_VER <= 1200)
-DECLARE_HANDLE(HDROP);
-#endif
-
 // Custom message event used for system tray handling
 #define PWS_MSG_ICON_NOTIFY             (WM_APP + 10)
 
@@ -242,9 +239,10 @@ public:
                  std::vector<int> &indices);
   size_t FindAll(const CString &str, BOOL CaseSensitive,
                  std::vector<int> &indices,
-                 const CItemData::FieldBits &bsFields, const bool &subgroup_set, 
-                 const std::wstring &subgroup_name, const int subgroup_object,
-                 const int subgroup_function);
+                 const CItemData::FieldBits &bsFields, 
+                 const CItemAtt::AttFieldBits &bsAttFields, 
+                 const bool &subgroup_set, const std::wstring &subgroup_name,
+                 const int subgroup_object, const int subgroup_function);
 
   // Used by ListCtrl KeyDown
   bool IsImageVisible() const {return m_bImageInLV;}
@@ -326,7 +324,11 @@ public:
   void FixListIndexes();
   void Delete(); // "Top level" delete, calls the following 2 and Execute()
   Command *Delete(const CItemData *pci); // create command for deleting a single item
-  Command *Delete(HTREEITEM ti); // For deleting a group
+  // For deleting a group:
+  void Delete(HTREEITEM ti,
+              std::vector<Command *> &vbases,
+              std::vector<Command *> &vdeps,
+              std::vector<Command *> &vemptygrps); 
 
   void SaveGroupDisplayState(); // call when tree expansion state changes
   void RestoreGUIStatusEx();
@@ -351,7 +353,7 @@ public:
   bool IsFilterActive() const {return m_bFilterActive;}
   int GetNumPassedFiltering() const {return m_bNumPassedFiltering;}
   CItemData *GetLastSelected() const;
-  StringX GetGroupName() const;
+  StringX GetGroupName(const bool bFullPath = false) const;
   void UpdateGroupNamesInMap(const StringX sxOldPath, const StringX sxNewPath);
 
   void SetFilter(FilterPool selectedpool, CString selectedfiltername)
@@ -413,22 +415,25 @@ public:
                  const bool bAdvanced, CReport *prpt, bool *pbCancel);
   void DoSynchronize(PWScore *pothercore,
                      const bool bAdvanced, int &numUpdated, CReport *prpt, bool *pbCancel);
-  int DoExportText(const StringX &sx_Filename, const bool bAll,
+  int DoExportText(const StringX &sx_Filename, const UINT nID,
                    const wchar_t &delimiter, const bool bAdvanced,
                    int &numExported, CReport *prpt);
-  int DoExportXML(const StringX &sx_Filename, const bool bAll,
+  int DoExportXML(const StringX &sx_Filename, const UINT nID,
                   const wchar_t &delimiter, const bool bAdvanced,
                   int &numExported, CReport *prpt);
 
+  int DoExportDB(const StringX &sx_Filename, const UINT nID,
+                 const StringX &sx_ExportKey, int &numExported, CReport *prpt);
+
   int TestSelection(const bool bAdvanced,
                     const stringT &subgroup_name,
                     const int &subgroup_object,
                     const int &subgroup_function,
-                    const OrderedItemList *il) const
+                    const OrderedItemList *pOIL) const
   {return m_core.TestSelection(bAdvanced, subgroup_name,
-                               subgroup_object, subgroup_function, il);}
+                               subgroup_object, subgroup_function, pOIL);}
 
-  void MakeOrderedItemList(OrderedItemList &il) const;
+  void MakeOrderedItemList(OrderedItemList &OIL, HTREEITEM hItem = NULL);
   bool MakeMatchingGTUSet(GTUSet &setGTU, const StringX &sxPolicyName) const
   {return m_core.InitialiseGTU(setGTU, sxPolicyName);}
   CItemData *getSelectedItem();
@@ -487,17 +492,24 @@ public:
   const unsigned int GetMenuShortcut(const unsigned short int &siVirtKey,
                                      const unsigned char &cModifier, StringX &sxMenuItemName);
   
-  // ClassWizard generated virtual function overrides
-  //{{AFX_VIRTUAL(DboxMain)
-
   void ChangeMode(bool promptUser); // r-o <-> r/w
 
   // If we have processed it returns 0 else 1
   BOOL ProcessEntryShortcut(WORD &wVirtualKeyCode, WORD &wModifiers);
   bool IsWorkstationLocked() const;
+
+  std::set<StringX> GetAllMediaTypes() const
+  {return m_core.GetAllMediaTypes();}
+
  protected:
-  virtual void DoDataExchange(CDataExchange* pDX);  // DDX/DDV support
-  //}}AFX_VIRTUAL
+   // ClassWizard generated virtual function overrides
+   //{{AFX_VIRTUAL(DboxMain)
+   virtual BOOL OnInitDialog();
+   virtual void OnCancel();
+   virtual void DoDataExchange(CDataExchange* pDX);  // DDX/DDV support
+   // override following to reset idle timeout on any event
+   virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
+   //}}AFX_VIRTUAL
 
   HICON m_hIcon;
   HICON m_hIconSm;
@@ -559,22 +571,17 @@ public:
   int OnUpdateViewReports(const int nID);
   void OnUpdateMRU(CCmdUI* pCmdUI);
 
-  // override following to reset idle timeout on any event
-  virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
-
   void ConfigureSystemMenu();
-  afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
+
+  // 'STATE' also defined in ThisMfcApp.h - ensure identical
+  enum STATE { LOCKED, UNLOCKED, CLOSED };
+  void UpdateSystemTray(const STATE s);
+
   LRESULT OnHotKey(WPARAM wParam, LPARAM lParam);
   LRESULT OnCCToHdrDragComplete(WPARAM wParam, LPARAM lParam);
   LRESULT OnHdrToCCDragComplete(WPARAM wParam, LPARAM lParam);
   LRESULT OnHeaderDragComplete(WPARAM wParam, LPARAM lParam);
-
-  // 'STATE' also defined in ThisMfcApp.h - ensure identical
-  enum STATE {LOCKED, UNLOCKED, CLOSED};
-  void UpdateSystemTray(const STATE s);
-
   LRESULT OnTrayNotification(WPARAM wParam, LPARAM lParam);
-
   LRESULT OnProcessCompareResultFunction(WPARAM wParam, LPARAM lParam);
   LRESULT OnProcessCompareResultAllFunction(WPARAM wParam, LPARAM lParam);
   LRESULT OnEditExpiredPasswordEntry(WPARAM wParam, LPARAM lParam);
@@ -586,7 +593,6 @@ public:
                              const pws_os::CUUID &fromuuid, const pws_os::CUUID &touuid);
   LRESULT CopyAllCompareResult(WPARAM wParam);
   LRESULT SynchAllCompareResult(WPARAM wParam);
-  
   LRESULT OnToolBarFindMessage(WPARAM wParam, LPARAM lParam);
   LRESULT OnDragAutoType(WPARAM wParam, LPARAM lParam);
   LRESULT OnExecuteFilters(WPARAM wParam, LPARAM lParam);
@@ -634,10 +640,17 @@ public:
                           const st_filters &filters) const;
   bool PassesPWPFiltering(const CItemData *pci,
                           const st_filters &filters) const;
+  bool PassesAttFiltering(const CItemData *pci,
+                          const st_filters &filters) const;
 
   void SaveGUIStatus();
   void RestoreGUIStatus();
 
+  void ChangeFont(const CFontsDialog::FontType iType);
+
+  // Generated message map functions
+  //{{AFX_MSG(DboxMain)
+  afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
   afx_msg void OnTrayLockUnLock();
   afx_msg void OnTrayClearRecentEntries();
   afx_msg void OnUpdateTrayClearRecentEntries(CCmdUI *pCmdUI);
@@ -664,11 +677,7 @@ public:
   afx_msg void OnTraySelect(UINT nID);
   afx_msg void OnUpdateTraySelect(CCmdUI *pCmdUI);
 
-  // Generated message map functions
-  //{{AFX_MSG(DboxMain)
-  virtual BOOL OnInitDialog();
-  virtual void OnCancel();
-  
+
   afx_msg LRESULT OnAreYouMe(WPARAM, LPARAM);
   afx_msg LRESULT OnWH_SHELL_CallBack(WPARAM wParam, LPARAM lParam);
   afx_msg LRESULT OnSessionChange(WPARAM wParam, LPARAM lParam);
@@ -745,6 +754,7 @@ public:
   afx_msg void OnCollapseAll();
   afx_msg void OnChangeTreeFont();
   afx_msg void OnChangePswdFont();
+  afx_msg void OnChangeNotesFont();
   afx_msg void OnChangeVKFont();
   afx_msg void OnViewReportsByID(UINT nID);  // From View->Reports menu
   afx_msg void OnViewReports();
@@ -770,14 +780,19 @@ public:
   afx_msg void OnUpdateNSCommand(CCmdUI *pCmdUI);  // Make entry unsupported (grayed out)
   afx_msg void OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);
   afx_msg void OnSizing(UINT fwSide, LPRECT pRect);
-  //}}AFX_MSG
 
   afx_msg BOOL OnToolTipText(UINT, NMHDR *pNotifyStruct, LRESULT *pLResult);
   afx_msg void OnExportVx(UINT nID);
+
   afx_msg void OnExportText();
   afx_msg void OnExportXML();
   afx_msg void OnExportEntryText();
   afx_msg void OnExportEntryXML();
+  afx_msg void OnExportEntryDB();
+  afx_msg void OnExportGroupText();
+  afx_msg void OnExportGroupXML();
+  afx_msg void OnExportGroupDB();
+  afx_msg void OnExportAttachment();
   afx_msg void OnImportText();
   afx_msg void OnImportKeePassV1CSV();
   afx_msg void OnImportKeePassV1TXT();
@@ -794,12 +809,8 @@ public:
 
   afx_msg void OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct);
   afx_msg void OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct);
-
-#if _MFC_VER > 1200
   afx_msg BOOL OnOpenMRU(UINT nID);
-#else
-  afx_msg void OnOpenMRU(UINT nID);
-#endif
+  //}}AFX_MSG
 
   DECLARE_MESSAGE_MAP()
 
@@ -835,13 +846,14 @@ private:
   bool m_bPasswordColumnShowing;
   bool m_bInRefresh, m_bInRestoreWindows;
   bool m_bDBInitiallyRO;
+  bool m_bViaDCA;
   int m_iDateTimeFieldWidth;
   int m_nColumns;
-  int m_nColumnIndexByOrder[CItemData::LAST];
-  int m_nColumnIndexByType[CItemData::LAST];
-  int m_nColumnTypeByIndex[CItemData::LAST];
-  int m_nColumnWidthByIndex[CItemData::LAST];
-  int m_nColumnHeaderWidthByType[CItemData::LAST];
+  int m_nColumnIndexByOrder[CItem::LAST_DATA];
+  int m_nColumnIndexByType[CItem::LAST_DATA];
+  int m_nColumnTypeByIndex[CItem::LAST_DATA];
+  int m_nColumnWidthByIndex[CItem::LAST_DATA];
+  int m_nColumnHeaderWidthByType[CItem::LAST_DATA];
   int m_iheadermaxwidth;
 
   pws_os::CUUID m_LUUIDSelectedAtMinimize; // to restore List entry selection upon un-minimize
@@ -866,7 +878,7 @@ private:
   bool LockDataBase();
   void startLockCheckTimer();
   UINT m_IdleLockCountDown;
-  void SetIdleLockCounter(UINT i); // i in minutes, set to timer counts
+  void SetIdleLockCounter(UINT iMinutes); // set to timer counts
   bool DecrementAndTestIdleLockCounter();
   int SaveIfChanged();
   void CheckExpireList(const bool bAtOpen = false); // Upon open, timer + menu, check list, show exp.
@@ -935,6 +947,7 @@ private:
   vfiltergroups m_vMflgroups;
   vfiltergroups m_vHflgroups;
   vfiltergroups m_vPflgroups;
+  vfiltergroups m_vAflgroups;
 
   // Global Filters
   PWSFilters m_MapFilters;
