@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -16,6 +16,7 @@
 
 #include "core/PwsPlatform.h"
 #include "core/PWSprefs.h"
+#include "os/env.h"
 
 #include "resource.h"
 #include "resource3.h"  // String resources
@@ -95,8 +96,9 @@ BOOL COptionsSecurity::OnInitDialog()
   }
 
   OnLockOnIdleTimeout();
-  CSpinButtonCtrl* pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_IDLESPIN);
+  CSpinButtonCtrl *pspin;
 
+  pspin = (CSpinButtonCtrl *)GetDlgItem(IDC_IDLESPIN);
   pspin->SetBuddy(GetDlgItem(IDC_IDLE_TIMEOUT));
   pspin->SetRange(1, 120);
   pspin->SetBase(10);
@@ -104,9 +106,10 @@ BOOL COptionsSecurity::OnInitDialog()
 
   CSliderCtrl *pslider = (CSliderCtrl *)GetDlgItem(IDC_HASHITERSLIDER);
   pslider->SetRange(MinHIslider, MaxHIslider);
+  pslider->SetTicFreq(1);
+  pslider->SetPos(m_HashIterSliderValue);
 
-  return TRUE;  // return TRUE unless you set the focus to a control
-  // EXCEPTION: OCX Property Pages should return FALSE
+  return TRUE;
 }
 
 void COptionsSecurity::UpdateHashIter()
