@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -52,11 +52,6 @@
 #include <errno.h>
 #include <io.h>
 
-// Only produce minidumps in release code
-#ifndef _DEBUG
-#include "Dbghelp.h"
-#endif
-
 using namespace std;
 
 #ifdef _DEBUG
@@ -673,6 +668,8 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
    * I think I'll keep the old functionality, but activate it with a "-e" or "-d" flag. (ronys)
    */
 
+  int dialogOrientation = -1; // update pref only if set
+
   allDone = false;
   if (m_lpCmdLine[0] != L'\0') {
     CString args = m_lpCmdLine;
@@ -732,6 +729,12 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
            * Supported natively in List View
            */
           dbox.AllowCompareEntries();
+        } else if ((*arg) == L"--do-tall") {
+          dialogOrientation = PWSprefs::TALL;
+        } else if ((*arg) == L"--do-wide") {
+          dialogOrientation = PWSprefs::WIDE;
+        } else if ((*arg) == L"--do-auto") {
+          dialogOrientation = PWSprefs::AUTO;
         } else {
           // unrecognized extended flag. Silently ignore.
         }
@@ -872,6 +875,15 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
     if (startSilent && !fileGiven)
       dbox.SetStartClosed(true);
   } // Command line not empty
+
+  if (!allDone) {
+    // MUST (indirectly) create PWSprefs first thing after parsing argv
+    // Ensures all things like saving locations etc. are set up.
+    PWSprefs *prefs = PWSprefs::GetInstance();
+    // Update dialog orientation preference if user specified:
+    if (dialogOrientation != -1)
+      prefs->SetPref(PWSprefs::DlgOrientation, dialogOrientation);
+  } // !allDone
   return true;
 }
 
@@ -926,9 +938,6 @@ BOOL ThisMfcApp::InitInstance()
   else if (!parseVal) // bad command line args
     return FALSE;
 
-  // MUST (indirectly) create PWSprefs first
-  // Ensures all things like saving locations etc. are set up.
-
   PWSprefs *prefs = PWSprefs::GetInstance();
 
   // And the others - even if not referenced here
@@ -1370,7 +1379,9 @@ void ThisMfcApp::GetLanguageFiles()
   // Add default embedded language - English
   const LCID AppLCID = MAKELCID(m_AppLangID, SORT_DEFAULT); // 0x0409 - English US
 
-  wchar_t *szLanguage_Native(NULL);
+  wchar_t *szLanguage_Native(NULL), *szLanguage_English(NULL);
+  wchar_t *szCountry_Native(NULL), *szCountry_English(NULL);
+
   int inum = ::GetLocaleInfo(AppLCID, LOCALE_SNATIVELANGNAME, szLanguage_Native, 0);
   if (inum > 0) {
     szLanguage_Native = new wchar_t[inum + 1];
@@ -1422,10 +1433,7 @@ void ThisMfcApp::GetLanguageFiles()
     // Create LCID
     LCID lcid = MAKELCID(FoundResLangID, SORT_DEFAULT);
 
-    wchar_t *szLanguage_Native(NULL), *szLanguage_English(NULL);
-    wchar_t *szCountry_Native(NULL), *szCountry_English(NULL);
-
-    int inum = ::GetLocaleInfo(lcid, LOCALE_SNATIVELANGNAME, NULL, 0);
+    inum = ::GetLocaleInfo(lcid, LOCALE_SNATIVELANGNAME, NULL, 0);
     if (inum > 0) {
       // Get language name in that language
       szLanguage_Native = new wchar_t[inum + 1];
